@@ -29,70 +29,68 @@
 */
 
 
+#include <cmath>
 #include <cstdio>
 #include <cstdlib>
-#include <cmath>
 #include <iostream>
 #include <iterator>
-#include <vector>
 #include <string>
+#include <vector>
 
-#include <boost/tokenizer.hpp>
-#include <boost/foreach.hpp>
-#include <boost/filesystem.hpp>
-
-#include "argparse.h"
-#include "imageio.h"
-#include "sysutil.h"
-#include "filesystem.h"
+#include <OpenImageIO/argparse.h>
+#include <OpenImageIO/filesystem.h>
+#include <OpenImageIO/imagecache.h>
+#include <OpenImageIO/imageio.h>
+#include <OpenImageIO/sysutil.h>
 
 
-OIIO_NAMESPACE_USING;
+using namespace OIIO;
 
 
-static std::string uninitialized = "uninitialized \001 HHRU dfvAS: efjl";
+static std::string uninitialized  = "uninitialized \001 HHRU dfvAS: efjl";
 static std::string dataformatname = "";
-static float gammaval = 1.0f;
+static float gammaval             = 1.0f;
 //static bool depth = false;
 static bool verbose = false;
-static int nthreads = 0;    // default: use #cores threads if available
+static int nthreads = 0;  // default: use #cores threads if available
 static std::vector<std::string> filenames;
-static int tile[3] = { 0, 0, 1 };
+static int tile[3]   = { 0, 0, 1 };
 static bool scanline = false;
 //static bool zfile = false;
 //static std::string channellist;
 static std::string compression;
-static bool no_copy_image = false;
-static int quality = -1;
-static bool adjust_time = false;
+static bool no_copy_image  = false;
+static int quality         = -1;
+static bool adjust_time    = false;
 static std::string caption = uninitialized;
 static std::vector<std::string> keywords;
 static bool clear_keywords = false;
 static std::vector<std::string> attribnames, attribvals;
-static bool inplace = false;
+static bool inplace    = false;
 static int orientation = 0;
 static bool rotcw = false, rotccw = false, rot180 = false;
-static bool sRGB = false;
+static bool sRGB     = false;
 static bool separate = false, contig = false;
 static bool noclobber = false;
 
 
 
 static int
-parse_files (int argc, const char *argv[])
+parse_files(int argc, const char* argv[])
 {
-    for (int i = 0;  i < argc;  i++)
-        filenames.push_back (argv[i]);
+    for (int i = 0; i < argc; i++)
+        filenames.emplace_back(argv[i]);
     return 0;
 }
 
 
 
 static void
-getargs (int argc, char *argv[])
+getargs(int argc, char* argv[])
 {
     bool help = false;
     ArgParse ap;
+    // clang-format off
     ap.options ("iconvert -- copy images with format conversions and other alterations\n"
                 OIIO_INTRO_STRING "\n"
                 "Usage:  iconvert [options] inputfile outputfile\n"
@@ -125,193 +123,160 @@ getargs (int argc, char *argv[])
                 "--no-clobber", &noclobber, "Do no overwrite existing files",
 //FIXME         "-z", &zfile, "Treat input as a depth file",
 //FIXME         "-c %s", &channellist, "Restrict/shuffle channels",
-                NULL);
+                nullptr);
+    // clang-format on
     if (ap.parse(argc, (const char**)argv) < 0) {
-	std::cerr << ap.geterror() << std::endl;
-        ap.usage ();
-        exit (EXIT_FAILURE);
+        std::cerr << ap.geterror() << std::endl;
+        ap.usage();
+        exit(EXIT_FAILURE);
     }
     if (help) {
-        ap.usage ();
-        exit (EXIT_FAILURE);
+        ap.usage();
+        exit(EXIT_FAILURE);
     }
 
-    if (filenames.size() != 2 && ! inplace) {
-        std::cerr << "iconvert: Must have both an input and output filename specified.\n";
+    if (filenames.size() != 2 && !inplace) {
+        std::cerr
+            << "iconvert: Must have both an input and output filename specified.\n";
         ap.usage();
-        exit (EXIT_FAILURE);
+        exit(EXIT_FAILURE);
     }
     if (filenames.size() == 0 && inplace) {
         std::cerr << "iconvert: Must have at least one filename\n";
         ap.usage();
-        exit (EXIT_FAILURE);
+        exit(EXIT_FAILURE);
     }
-    if (((int)rotcw + (int)rotccw + (int)rot180 + (orientation>0)) > 1) {
-        std::cerr << "iconvert: more than one of --rotcw, --rotccw, --rot180, --orientation\n";
+    if (((int)rotcw + (int)rotccw + (int)rot180 + (orientation > 0)) > 1) {
+        std::cerr
+            << "iconvert: more than one of --rotcw, --rotccw, --rot180, --orientation\n";
         ap.usage();
-        exit (EXIT_FAILURE);
+        exit(EXIT_FAILURE);
     }
 }
 
 
 
 static bool
-DateTime_to_time_t (const char *datetime, time_t &timet)
+DateTime_to_time_t(const char* datetime, time_t& timet)
 {
     int year, month, day, hour, min, sec;
-    int r = sscanf (datetime, "%d:%d:%d %d:%d:%d",
-                    &year, &month, &day, &hour, &min, &sec);
+    int r = sscanf(datetime, "%d:%d:%d %d:%d:%d", &year, &month, &day, &hour,
+                   &min, &sec);
     // printf ("%d  %d:%d:%d %d:%d:%d\n", r, year, month, day, hour, min, sec);
     if (r != 6)
         return false;
     struct tm tmtime;
     time_t now;
-    Sysutil::get_local_time (&now, &tmtime); // fill in defaults
-    tmtime.tm_sec = sec;
-    tmtime.tm_min = min;
+    Sysutil::get_local_time(&now, &tmtime);  // fill in defaults
+    tmtime.tm_sec  = sec;
+    tmtime.tm_min  = min;
     tmtime.tm_hour = hour;
     tmtime.tm_mday = day;
-    tmtime.tm_mon = month-1;
-    tmtime.tm_year = year-1900;
-    timet = mktime (&tmtime);
+    tmtime.tm_mon  = month - 1;
+    tmtime.tm_year = year - 1900;
+    timet          = mktime(&tmtime);
     return true;
 }
 
 
 
-// Utility: split semicolon-separated list
-static void
-split_list (const std::string &list, std::vector<std::string> &items)
-{
-    typedef boost::tokenizer<boost::char_separator<char> > tokenizer;
-    boost::char_separator<char> sep(";");
-    tokenizer tokens (list, sep);
-    for (tokenizer::iterator tok_iter = tokens.begin();
-         tok_iter != tokens.end(); ++tok_iter) {
-        std::string t = *tok_iter;
-        while (t.length() && t[0] == ' ')
-            t.erase (t.begin());
-        if (t.length())
-            items.push_back (t);
-    }
-}
-
-
-
-// Utility: join list into a single semicolon-separated string
-static std::string
-join_list (const std::vector<std::string> &items)
-{
-    std::string s;
-    for (size_t i = 0;  i < items.size();  ++i) {
-        if (i > 0)
-            s += "; ";
-        s += items[i];
-    }
-    return s;
-}
-
-
-
 // Adjust the output spec based on the command-line arguments.
 // Return whether the specifics preclude using copy_image.
 static bool
-adjust_spec (ImageInput *in, ImageOutput *out,
-             const ImageSpec &inspec, ImageSpec &outspec)
+adjust_spec(ImageInput* in, ImageOutput* out, const ImageSpec& inspec,
+            ImageSpec& outspec)
 {
     bool nocopy = no_copy_image;
 
     // Copy the spec, with possible change in format
-    outspec.set_format (inspec.format);
+    outspec.format = inspec.format;
     if (inspec.channelformats.size()) {
         // Input file has mixed channels
         if (out->supports("channelformats")) {
             // Output supports mixed formats -- so request it
-            outspec.set_format (TypeDesc::UNKNOWN);
+            outspec.format = TypeDesc::UNKNOWN;
         } else {
             // Input had mixed formats, output did not, so just use a fixed
             // format and forget the per-channel formats for output.
-            outspec.channelformats.clear ();
+            outspec.channelformats.clear();
         }
     }
-    if (! dataformatname.empty()) {
+    if (!dataformatname.empty()) {
         // make sure there isn't a stray BPS that will screw us up
-        outspec.erase_attribute ("oiio:BitsPerSample");
+        outspec.erase_attribute("oiio:BitsPerSample");
         if (dataformatname == "uint8")
-            outspec.set_format (TypeDesc::UINT8);
+            outspec.set_format(TypeDesc::UINT8);
         else if (dataformatname == "int8")
-            outspec.set_format (TypeDesc::INT8);
+            outspec.set_format(TypeDesc::INT8);
         else if (dataformatname == "uint10") {
-            outspec.attribute ("oiio:BitsPerSample", 10);
-            outspec.set_format (TypeDesc::UINT16);
-        }
-        else if (dataformatname == "uint12") {
-            outspec.attribute ("oiio:BitsPerSample", 12);
-            outspec.set_format (TypeDesc::UINT16);
-        }
-        else if (dataformatname == "uint16")
-            outspec.set_format (TypeDesc::UINT16);
+            outspec.attribute("oiio:BitsPerSample", 10);
+            outspec.set_format(TypeDesc::UINT16);
+        } else if (dataformatname == "uint12") {
+            outspec.attribute("oiio:BitsPerSample", 12);
+            outspec.set_format(TypeDesc::UINT16);
+        } else if (dataformatname == "uint16")
+            outspec.set_format(TypeDesc::UINT16);
         else if (dataformatname == "int16")
-            outspec.set_format (TypeDesc::INT16);
+            outspec.set_format(TypeDesc::INT16);
         else if (dataformatname == "uint32" || dataformatname == "uint")
-            outspec.set_format (TypeDesc::UINT32);
+            outspec.set_format(TypeDesc::UINT32);
         else if (dataformatname == "int32" || dataformatname == "int")
-            outspec.set_format (TypeDesc::INT32);
+            outspec.set_format(TypeDesc::INT32);
         else if (dataformatname == "half")
-            outspec.set_format (TypeDesc::HALF);
+            outspec.set_format(TypeDesc::HALF);
         else if (dataformatname == "float")
-            outspec.set_format (TypeDesc::FLOAT);
+            outspec.set_format(TypeDesc::FLOAT);
         else if (dataformatname == "double")
-            outspec.set_format (TypeDesc::DOUBLE);
-        outspec.channelformats.clear ();
+            outspec.set_format(TypeDesc::DOUBLE);
+        outspec.channelformats.clear();
     }
     if (outspec.format != inspec.format || inspec.channelformats.size())
         nocopy = true;
-    
-    outspec.attribute ("oiio:Gamma", gammaval);
+
+    outspec.attribute("oiio:Gamma", gammaval);
     if (sRGB) {
-        outspec.attribute ("oiio:ColorSpace", "sRGB");
-        if (!strcmp (in->format_name(), "jpeg") ||
-                outspec.find_attribute ("Exif:ColorSpace"))
-            outspec.attribute ("Exif:ColorSpace", 1);
+        outspec.attribute("oiio:ColorSpace", "sRGB");
+        if (!strcmp(in->format_name(), "jpeg")
+            || outspec.find_attribute("Exif:ColorSpace"))
+            outspec.attribute("Exif:ColorSpace", 1);
     }
 
     if (tile[0]) {
-        outspec.tile_width = tile[0];
+        outspec.tile_width  = tile[0];
         outspec.tile_height = tile[1];
-        outspec.tile_depth = tile[2];
+        outspec.tile_depth  = tile[2];
     }
     if (scanline) {
-        outspec.tile_width = 0;
+        outspec.tile_width  = 0;
         outspec.tile_height = 0;
-        outspec.tile_depth = 0;
+        outspec.tile_depth  = 0;
     }
-    if (outspec.tile_width != inspec.tile_width ||
-            outspec.tile_height != inspec.tile_height ||
-            outspec.tile_depth != inspec.tile_depth)
+    if (outspec.tile_width != inspec.tile_width
+        || outspec.tile_height != inspec.tile_height
+        || outspec.tile_depth != inspec.tile_depth)
         nocopy = true;
 
-    if (! compression.empty()) {
-        outspec.attribute ("compression", compression);
-        if (compression != inspec.get_string_attribute ("compression"))
+    if (!compression.empty()) {
+        outspec.attribute("compression", compression);
+        if (compression != inspec.get_string_attribute("compression"))
             nocopy = true;
     }
 
     if (quality > 0) {
-        outspec.attribute ("CompressionQuality", quality);
-        if (quality != inspec.get_int_attribute ("CompressionQuality"))
+        outspec.attribute("CompressionQuality", quality);
+        if (quality != inspec.get_int_attribute("CompressionQuality"))
             nocopy = true;
     }
 
     if (contig)
-        outspec.attribute ("planarconfig", "contig");
+        outspec.attribute("planarconfig", "contig");
     if (separate)
-        outspec.attribute ("planarconfig", "separate");
+        outspec.attribute("planarconfig", "separate");
 
     if (orientation >= 1)
-        outspec.attribute ("Orientation", orientation);
+        outspec.attribute("Orientation", orientation);
     else {
-        orientation = outspec.get_int_attribute ("Orientation", 1);
+        orientation = outspec.get_int_attribute("Orientation", 1);
         if (orientation >= 1 && orientation <= 8) {
             static int cw[] = { 0, 6, 7, 8, 5, 2, 3, 4, 1 };
             if (rotcw || rotccw || rot180)
@@ -320,32 +285,35 @@ adjust_spec (ImageInput *in, ImageOutput *out,
                 orientation = cw[orientation];
             if (rotccw)
                 orientation = cw[orientation];
-            outspec.attribute ("Orientation", orientation);
+            outspec.attribute("Orientation", orientation);
         }
     }
 
     if (caption != uninitialized)
-        outspec.attribute ("ImageDescription", caption);
+        outspec.attribute("ImageDescription", caption);
 
     if (clear_keywords)
-        outspec.attribute ("Keywords", "");
+        outspec.attribute("Keywords", "");
     if (keywords.size()) {
-        std::string oldkw = outspec.get_string_attribute ("Keywords");
+        std::string oldkw = outspec.get_string_attribute("Keywords");
         std::vector<std::string> oldkwlist;
-        if (! oldkw.empty())
-            split_list (oldkw, oldkwlist);
-        BOOST_FOREACH (const std::string &nk, keywords) {
-            bool dup = false;
-            BOOST_FOREACH (const std::string &ok, oldkwlist)
-                dup |= (ok == nk);
-            if (! dup)
-                oldkwlist.push_back (nk);
+        if (!oldkw.empty()) {
+            Strutil::split(oldkw, oldkwlist, ";");
+            for (auto& kw : oldkwlist)
+                kw = Strutil::strip(kw);
         }
-        outspec.attribute ("Keywords", join_list (oldkwlist));
+        for (auto&& nk : keywords) {
+            bool dup = false;
+            for (auto&& ok : oldkwlist)
+                dup |= (ok == nk);
+            if (!dup)
+                oldkwlist.push_back(nk);
+        }
+        outspec.attribute("Keywords", Strutil::join(oldkwlist, "; "));
     }
 
-    for (size_t i = 0;  i < attribnames.size();  ++i) {
-        outspec.attribute (attribnames[i].c_str(), attribvals[i].c_str());
+    for (size_t i = 0; i < attribnames.size(); ++i) {
+        outspec.attribute(attribnames[i].c_str(), attribvals[i].c_str());
     }
 
     return nocopy;
@@ -354,7 +322,7 @@ adjust_spec (ImageInput *in, ImageOutput *out,
 
 
 static bool
-convert_file (const std::string &in_filename, const std::string &out_filename)
+convert_file(const std::string& in_filename, const std::string& out_filename)
 {
     if (noclobber && Filesystem::exists(out_filename)) {
         std::cerr << "iconvert ERROR: Output file already exists \""
@@ -363,49 +331,63 @@ convert_file (const std::string &in_filename, const std::string &out_filename)
     }
 
     if (verbose)
-        std::cout << "Converting " << in_filename << " to " << out_filename << "\n";
+        std::cout << "Converting " << in_filename << " to " << out_filename
+                  << "\n";
 
     std::string tempname = out_filename;
     if (tempname == in_filename) {
-        tempname = out_filename + ".tmp."
-                    + Filesystem::file_extension (out_filename);
+        tempname = out_filename + ".tmp" + Filesystem::extension(out_filename);
     }
 
     // Find an ImageIO plugin that can open the input file, and open it.
-    ImageInput *in = ImageInput::create (in_filename.c_str(), "" /* searchpath */);
-    if (! in) {
+    auto in = ImageInput::open(in_filename);
+    if (!in) {
         std::string err = geterror();
-        std::cerr << "iconvert ERROR: " 
-                  << (err.length() ? err : Strutil::format("Could not open \"%s\"", in_filename.c_str()))
+        std::cerr << "iconvert ERROR: "
+                  << (err.length() ? err
+                                   : Strutil::sprintf("Could not open \"%s\"",
+                                                      in_filename))
                   << "\n";
         return false;
     }
-    ImageSpec inspec;
-    if (! in->open (in_filename.c_str(), inspec)) {
-        std::cerr << "iconvert ERROR: Could not open \"" << in_filename
-                  << "\" : " << in->geterror() << "\n";
-        delete in;
-        return false;
-    }
-    std::string metadatatime = inspec.get_string_attribute ("DateTime");
+    ImageSpec inspec         = in->spec();
+    std::string metadatatime = inspec.get_string_attribute("DateTime");
 
     // Find an ImageIO plugin that can open the output file, and open it
-    ImageOutput *out = ImageOutput::create (tempname.c_str());
-    if (! out) {
-        std::cerr 
-            << "iconvert ERROR: Could not find an ImageIO plugin to write \"" 
+    auto out = ImageOutput::create(tempname);
+    if (!out) {
+        std::cerr
+            << "iconvert ERROR: Could not find an ImageIO plugin to write \""
             << out_filename << "\" :" << geterror() << "\n";
-        delete in;
         return false;
     }
 
-    bool ok = true;
-    bool mip_to_subimage_warning = false;
-    for (int subimage = 0;
-           ok && in->seek_subimage(subimage,0,inspec);
-           ++subimage) {
+    // In order to deal with formats that support subimages, but not
+    // subimage appending, we gather them all first.
+    std::vector<ImageSpec> subimagespecs;
+    if (out->supports("multiimage") && !out->supports("appendsubimage")) {
+        ImageCache* imagecache = ImageCache::create();
+        int nsubimages         = 0;
+        ustring ufilename(in_filename);
+        imagecache->get_image_info(ufilename, 0, 0, ustring("subimages"),
+                                   TypeInt, &nsubimages);
+        if (nsubimages > 1) {
+            subimagespecs.resize(nsubimages);
+            for (int i = 0; i < nsubimages; ++i) {
+                ImageSpec inspec = *imagecache->imagespec(ufilename, i, 0,
+                                                          true /*native*/);
+                subimagespecs[i] = inspec;
+                adjust_spec(in.get(), out.get(), inspec, subimagespecs[i]);
+            }
+        }
+        ImageCache::destroy(imagecache);
+    }
 
-        if (subimage > 0 &&  !out->supports ("multiimage")) {
+    bool ok                      = true;
+    bool mip_to_subimage_warning = false;
+    for (int subimage = 0; ok && in->seek_subimage(subimage, 0, inspec);
+         ++subimage) {
+        if (subimage > 0 && !out->supports("multiimage")) {
             std::cerr << "iconvert WARNING: " << out->format_name()
                       << " does not support multiple subimages.\n";
             std::cerr << "\tOnly the first subimage has been copied.\n";
@@ -416,19 +398,21 @@ convert_file (const std::string &in_filename, const std::string &out_filename)
         do {
             // Copy the spec, with possible change in format
             ImageSpec outspec = inspec;
-            bool nocopy = adjust_spec (in, out, inspec, outspec);
-        
-            ImageOutput::OpenMode mode;
+            bool nocopy = adjust_spec(in.get(), out.get(), inspec, outspec);
             if (miplevel > 0) {
-                if (out->supports ("mipmap"))
+                // Moving to next MIP level
+                ImageOutput::OpenMode mode;
+                if (out->supports("mipmap"))
                     mode = ImageOutput::AppendMIPLevel;
-                else if (out->supports ("multiimage")) {
-                    mode = ImageOutput::AppendSubimage; // use if we must
-                    if (! mip_to_subimage_warning
-                        && strcmp(out->format_name(),"tiff")) {
+                else if (out->supports("multiimage")
+                         && out->supports("appendsubimage")) {
+                    mode = ImageOutput::AppendSubimage;  // use if we must
+                    if (!mip_to_subimage_warning
+                        && strcmp(out->format_name(), "tiff")) {
                         std::cerr << "iconvert WARNING: " << out->format_name()
                                   << " does not support MIPmaps.\n";
-                        std::cerr << "\tStoring the MIPmap levels in subimages.\n";
+                        std::cerr
+                            << "\tStoring the MIPmap levels in subimages.\n";
                     }
                     mip_to_subimage_warning = true;
                 } else {
@@ -437,72 +421,80 @@ convert_file (const std::string &in_filename, const std::string &out_filename)
                     std::cerr << "\tOnly the first level has been copied.\n";
                     break;  // on to the next subimage
                 }
+                ok = out->open(tempname.c_str(), outspec, mode);
+            } else if (subimage > 0) {
+                // Moving to next subimage
+                ok = out->open(tempname.c_str(), outspec,
+                               ImageOutput::AppendSubimage);
+            } else {
+                // First time opening
+                if (subimagespecs.size())
+                    ok = out->open(tempname.c_str(), int(subimagespecs.size()),
+                                   &subimagespecs[0]);
+                else
+                    ok = out->open(tempname.c_str(), outspec,
+                                   ImageOutput::Create);
             }
-            else if (subimage > 0)
-                mode = ImageOutput::AppendSubimage;
-            else
-                mode = ImageOutput::Create;
-
-            if (! out->open (tempname.c_str(), outspec, mode)) {
-                std::string err = geterror();
-                std::cerr << "iconvert ERROR: " 
-                          << (err.length() ? err : Strutil::format("Could not open \"%s\"", out_filename.c_str()))
+            if (!ok) {
+                std::string err = out->geterror();
+                std::cerr << "iconvert ERROR: "
+                          << (err.length()
+                                  ? err
+                                  : Strutil::sprintf("Could not open \"%s\"",
+                                                     out_filename))
                           << "\n";
                 ok = false;
                 break;
             }
 
-            if (! nocopy) {
-                ok = out->copy_image (in);
-                if (! ok)
-                    std::cerr << "iconvert ERROR copying \"" << in_filename 
-                              << "\" to \"" << out_filename << "\" :\n\t" 
+            if (!nocopy) {
+                ok = out->copy_image(in.get());
+                if (!ok)
+                    std::cerr << "iconvert ERROR copying \"" << in_filename
+                              << "\" to \"" << out_filename << "\" :\n\t"
                               << out->geterror() << "\n";
             } else {
                 // Need to do it by hand for some reason.  Future expansion in which
                 // only a subset of channels are copied, or some such.
-                std::vector<char> pixels ((size_t)outspec.image_bytes(true));
-                ok = in->read_image (outspec.format, &pixels[0]);
-                if (! ok) {
-                    std::cerr << "iconvert ERROR reading \"" << in_filename 
+                std::vector<char> pixels((size_t)outspec.image_bytes(true));
+                ok = in->read_image(outspec.format, &pixels[0]);
+                if (!ok) {
+                    std::cerr << "iconvert ERROR reading \"" << in_filename
                               << "\" : " << in->geterror() << "\n";
                 } else {
-                    ok = out->write_image (outspec.format, &pixels[0]);
-                    if (! ok)
-                        std::cerr << "iconvert ERROR writing \"" << out_filename 
+                    ok = out->write_image(outspec.format, &pixels[0]);
+                    if (!ok)
+                        std::cerr << "iconvert ERROR writing \"" << out_filename
                                   << "\" : " << out->geterror() << "\n";
                 }
             }
-        
+
             ++miplevel;
-        } while (ok && in->seek_subimage(subimage,miplevel,inspec));
+        } while (ok && in->seek_subimage(subimage, miplevel, inspec));
     }
 
-    out->close ();
-    delete out;
-    in->close ();
-    delete in;
+    out->close();
+    in->close();
 
     // Figure out a time for the input file -- either one supplied by
     // the metadata, or the actual time stamp of the input file.
     std::time_t in_time;
-    if (metadatatime.empty() ||
-           ! DateTime_to_time_t (metadatatime.c_str(), in_time))
-        in_time = boost::filesystem::last_write_time (in_filename);
+    if (metadatatime.empty()
+        || !DateTime_to_time_t(metadatatime.c_str(), in_time))
+        in_time = Filesystem::last_write_time(in_filename);
 
     if (out_filename != tempname) {
         if (ok) {
-            boost::filesystem::remove (out_filename);
-            boost::filesystem::rename (tempname, out_filename);
-        }
-        else
-            boost::filesystem::remove (tempname);
+            Filesystem::remove(out_filename);
+            Filesystem::rename(tempname, out_filename);
+        } else
+            Filesystem::remove(tempname);
     }
 
     // If user requested, try to adjust the file's modification time to
     // the creation time indicated by the file's DateTime metadata.
     if (ok && adjust_time)
-        boost::filesystem::last_write_time (out_filename, in_time);
+        Filesystem::last_write_time(out_filename, in_time);
 
     return ok;
 }
@@ -510,19 +502,20 @@ convert_file (const std::string &in_filename, const std::string &out_filename)
 
 
 int
-main (int argc, char *argv[])
+main(int argc, char* argv[])
 {
-    getargs (argc, argv);
+    Filesystem::convert_native_arguments(argc, (const char**)argv);
+    getargs(argc, argv);
 
-    OIIO_NAMESPACE::attribute ("threads", nthreads);
+    OIIO::attribute("threads", nthreads);
 
     bool ok = true;
 
     if (inplace) {
-        BOOST_FOREACH (const std::string &s, filenames)
-            ok &= convert_file (s, s);
+        for (auto&& s : filenames)
+            ok &= convert_file(s, s);
     } else {
-        ok = convert_file (filenames[0], filenames[1]);
+        ok = convert_file(filenames[0], filenames[1]);
     }
 
     return ok ? EXIT_SUCCESS : EXIT_FAILURE;
