@@ -28,23 +28,26 @@
   (This is the Modified BSD License)
 */
 
-#ifndef OPENIMAGEIO_PNG_PVT_H
-#define OPENIMAGEIO_PNG_PVT_H
+#pragma once
 
 #include <png.h>
 #include <zlib.h>
+
 #include <OpenEXR/ImathColor.h>
 
 #include <OpenImageIO/dassert.h>
-#include <OpenImageIO/typedesc.h>
-#include <OpenImageIO/imageio.h>
-#include <OpenImageIO/strutil.h>
 #include <OpenImageIO/filesystem.h>
 #include <OpenImageIO/fmath.h>
+#include <OpenImageIO/imageio.h>
+#include <OpenImageIO/strutil.h>
 #include <OpenImageIO/sysutil.h>
+#include <OpenImageIO/tiffutils.h>
+#include <OpenImageIO/typedesc.h>
 
 
-#define OIIO_LIBPNG_VERSION (PNG_LIBPNG_VER_MAJOR*10000 + PNG_LIBPNG_VER_MINOR*100 + PNG_LIBPNG_VER_RELEASE)
+#define OIIO_LIBPNG_VERSION                                                    \
+    (PNG_LIBPNG_VER_MAJOR * 10000 + PNG_LIBPNG_VER_MINOR * 100                 \
+     + PNG_LIBPNG_VER_RELEASE)
 
 
 /*
@@ -71,18 +74,18 @@ namespace PNG_pvt {
 /// \return empty string on success, error message on failure.
 ///
 inline const std::string
-create_read_struct (png_structp& sp, png_infop& ip)
+create_read_struct(png_structp& sp, png_infop& ip)
 {
-    sp = png_create_read_struct (PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
-    if (! sp)
+    sp = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
+    if (!sp)
         return "Could not create PNG read structure";
 
-    ip = png_create_info_struct (sp);
-    if (! ip)
+    ip = png_create_info_struct(sp);
+    if (!ip)
         return "Could not create PNG info structure";
 
     // Must call this setjmp in every function that does PNG reads
-    if (setjmp (png_jmpbuf(sp)))
+    if (setjmp(png_jmpbuf(sp)))
         return "PNG library error";
 
     // success
@@ -94,31 +97,31 @@ create_read_struct (png_structp& sp, png_infop& ip)
 /// Helper function - reads background colour.
 ///
 inline bool
-get_background (png_structp& sp, png_infop& ip, ImageSpec& spec,
-                int& bit_depth, float *red, float *green, float *blue)
+get_background(png_structp& sp, png_infop& ip, ImageSpec& spec, int& bit_depth,
+               float* red, float* green, float* blue)
 {
-    if (setjmp (png_jmpbuf (sp)))
+    if (setjmp(png_jmpbuf(sp)))
         return false;
-    if (! png_get_valid (sp, ip, PNG_INFO_bKGD))
+    if (!png_get_valid(sp, ip, PNG_INFO_bKGD))
         return false;
 
     png_color_16p bg;
-    png_get_bKGD (sp, ip, &bg);
+    png_get_bKGD(sp, ip, &bg);
     if (spec.format == TypeDesc::UINT16) {
-        *red   = bg->red   / 65535.0;
+        *red   = bg->red / 65535.0;
         *green = bg->green / 65535.0;
-        *blue  = bg->blue  / 65535.0;
+        *blue  = bg->blue / 65535.0;
     } else if (spec.nchannels < 3 && bit_depth < 8) {
         if (bit_depth == 1)
             *red = *green = *blue = (bg->gray ? 1 : 0);
         else if (bit_depth == 2)
             *red = *green = *blue = bg->gray / 3.0;
-        else // 4 bits
+        else  // 4 bits
             *red = *green = *blue = bg->gray / 15.0;
     } else {
-        *red   = bg->red   / 255.0;
+        *red   = bg->red / 255.0;
         *green = bg->green / 255.0;
-        *blue  = bg->blue  / 255.0;
+        *blue  = bg->blue / 255.0;
     }
     return true;
 }
@@ -128,133 +131,135 @@ get_background (png_structp& sp, png_infop& ip, ImageSpec& spec,
 /// Read information from a PNG file and fill the ImageSpec accordingly.
 ///
 inline void
-read_info (png_structp& sp, png_infop& ip, int& bit_depth, int& color_type,
-           int& interlace_type, Imath::Color3f& bg, ImageSpec& spec,
-           bool keep_unassociated_alpha)
+read_info(png_structp& sp, png_infop& ip, int& bit_depth, int& color_type,
+          int& interlace_type, Imath::Color3f& bg, ImageSpec& spec,
+          bool keep_unassociated_alpha)
 {
-    png_read_info (sp, ip);
+    png_read_info(sp, ip);
 
     // Auto-convert 1-, 2-, and 4- bit images to 8 bits, palette to RGB,
     // and transparency to alpha.
-    png_set_expand (sp);
+    png_set_expand(sp);
 
     // PNG files are naturally big-endian
     if (littleendian())
-        png_set_swap (sp);
+        png_set_swap(sp);
+
+    png_read_update_info(sp, ip);
 
-    png_read_update_info (sp, ip);
-    
     png_uint_32 width, height;
-    png_get_IHDR (sp, ip, &width, &height,
-                  &bit_depth, &color_type, NULL, NULL, NULL);
-    
+    png_get_IHDR(sp, ip, &width, &height, &bit_depth, &color_type, NULL, NULL,
+                 NULL);
 
-    spec = ImageSpec ((int)width, (int)height, 
-                       png_get_channels (sp, ip),
-                       bit_depth == 16 ? TypeDesc::UINT16 : TypeDesc::UINT8);
 
-    spec.default_channel_names ();
+    spec = ImageSpec((int)width, (int)height, png_get_channels(sp, ip),
+                     bit_depth == 16 ? TypeDesc::UINT16 : TypeDesc::UINT8);
+
+    spec.default_channel_names();
     if (spec.nchannels == 2) {
         // Special case: PNG spec says 2-channel image is Gray & Alpha
         spec.channelnames[0] = "Y";
         spec.channelnames[1] = "A";
-        spec.alpha_channel = 1;
+        spec.alpha_channel   = 1;
     }
 
     int srgb_intent;
-    if (png_get_sRGB (sp, ip, &srgb_intent)) {
-        spec.attribute ("oiio:ColorSpace", "sRGB");
+    if (png_get_sRGB(sp, ip, &srgb_intent)) {
+        spec.attribute("oiio:ColorSpace", "sRGB");
     } else {
         double gamma;
-        if (png_get_gAMA (sp, ip, &gamma)) {
+        if (png_get_gAMA(sp, ip, &gamma)) {
             // Round gamma to the nearest hundredth to prevent stupid
             // precision choices and make it easier for apps to make
             // decisions based on known gamma values. For example, you want
             // 2.2, not 2.19998.
-            float g = float (1.0 / gamma);
-            g = roundf (100.0 * g) / 100.0f;
-            spec.attribute ("oiio:Gamma", g);
+            float g = float(1.0 / gamma);
+            g       = roundf(100.0 * g) / 100.0f;
+            spec.attribute("oiio:Gamma", g);
             if (g == 1.0f)
-                spec.attribute ("oiio:ColorSpace", "linear");
+                spec.attribute("oiio:ColorSpace", "linear");
             else
-                spec.attribute ("oiio:ColorSpace",
-                                Strutil::format("GammaCorrected%.2g", g));
+                spec.attribute("oiio:ColorSpace",
+                               Strutil::sprintf("GammaCorrected%.2g", g));
         }
     }
 
     if (png_get_valid(sp, ip, PNG_INFO_iCCP)) {
         png_charp profile_name = NULL;
-#if OIIO_LIBPNG_VERSION > 10500   /* PNG function signatures changed */
+#if OIIO_LIBPNG_VERSION > 10500 /* PNG function signatures changed */
         png_bytep profile_data = NULL;
 #else
         png_charp profile_data = NULL;
 #endif
         png_uint_32 profile_length = 0;
         int compression_type;
-        png_get_iCCP (sp, ip, &profile_name, &compression_type,
-                      &profile_data, &profile_length);
+        png_get_iCCP(sp, ip, &profile_name, &compression_type, &profile_data,
+                     &profile_length);
         if (profile_length && profile_data)
-            spec.attribute (ICC_PROFILE_ATTR, TypeDesc(TypeDesc::UINT8, profile_length), profile_data);
+            spec.attribute(ICC_PROFILE_ATTR,
+                           TypeDesc(TypeDesc::UINT8, profile_length),
+                           profile_data);
     }
 
     png_timep mod_time;
-    if (png_get_tIME (sp, ip, &mod_time)) {
-        std::string date = Strutil::format ("%4d:%02d:%02d %02d:%02d:%02d",
-                           mod_time->year, mod_time->month, mod_time->day,
-                           mod_time->hour, mod_time->minute, mod_time->second);
-        spec.attribute ("DateTime", date); 
+    if (png_get_tIME(sp, ip, &mod_time)) {
+        std::string date = Strutil::sprintf("%4d:%02d:%02d %02d:%02d:%02d",
+                                            mod_time->year, mod_time->month,
+                                            mod_time->day, mod_time->hour,
+                                            mod_time->minute, mod_time->second);
+        spec.attribute("DateTime", date);
     }
-    
+
     png_textp text_ptr;
-    int num_comments = png_get_text (sp, ip, &text_ptr, NULL);
+    int num_comments = png_get_text(sp, ip, &text_ptr, NULL);
     if (num_comments) {
         std::string comments;
-        for (int i = 0;  i < num_comments;  ++i) {
-            if (Strutil::iequals (text_ptr[i].key, "Description"))
-                spec.attribute ("ImageDescription", text_ptr[i].text);
-            else if (Strutil::iequals (text_ptr[i].key, "Author"))
-                spec.attribute ("Artist", text_ptr[i].text);
-            else if (Strutil::iequals (text_ptr[i].key, "Title"))
-                spec.attribute ("DocumentName", text_ptr[i].text);
-            else if (Strutil::iequals (text_ptr[i].key, "XML:com.adobe.xmp"))
-                decode_xmp (text_ptr[i].text, spec);
+        for (int i = 0; i < num_comments; ++i) {
+            if (Strutil::iequals(text_ptr[i].key, "Description"))
+                spec.attribute("ImageDescription", text_ptr[i].text);
+            else if (Strutil::iequals(text_ptr[i].key, "Author"))
+                spec.attribute("Artist", text_ptr[i].text);
+            else if (Strutil::iequals(text_ptr[i].key, "Title"))
+                spec.attribute("DocumentName", text_ptr[i].text);
+            else if (Strutil::iequals(text_ptr[i].key, "XML:com.adobe.xmp"))
+                decode_xmp(text_ptr[i].text, spec);
             else
-                spec.attribute (text_ptr[i].key, text_ptr[i].text);
+                spec.attribute(text_ptr[i].key, text_ptr[i].text);
         }
     }
-    spec.x = png_get_x_offset_pixels (sp, ip);
-    spec.y = png_get_y_offset_pixels (sp, ip);
+    spec.x = png_get_x_offset_pixels(sp, ip);
+    spec.y = png_get_y_offset_pixels(sp, ip);
 
     int unit;
     png_uint_32 resx, resy;
-    if (png_get_pHYs (sp, ip, &resx, &resy, &unit)) {
+    if (png_get_pHYs(sp, ip, &resx, &resy, &unit)) {
         float scale = 1;
         if (unit == PNG_RESOLUTION_METER) {
             // Convert to inches, to match most other formats
             scale = 2.54 / 100.0;
-            spec.attribute ("ResolutionUnit", "inch");
+            spec.attribute("ResolutionUnit", "inch");
         } else
-            spec.attribute ("ResolutionUnit", "none");
-        spec.attribute ("XResolution", (float)resx*scale);
-        spec.attribute ("YResolution", (float)resy*scale);
+            spec.attribute("ResolutionUnit", "none");
+        spec.attribute("XResolution", (float)resx * scale);
+        spec.attribute("YResolution", (float)resy * scale);
     }
 
-    float aspect = (float)png_get_pixel_aspect_ratio (sp, ip);
+    float aspect = (float)png_get_pixel_aspect_ratio(sp, ip);
     if (aspect != 0 && aspect != 1)
-        spec.attribute ("PixelAspectRatio", aspect);
+        spec.attribute("PixelAspectRatio", aspect);
 
     float r, g, b;
-    if (get_background (sp, ip, spec, bit_depth, &r, &g, &b)) {
-        bg = Imath::Color3f (r, g, b);
+    if (get_background(sp, ip, spec, bit_depth, &r, &g, &b)) {
+        bg = Imath::Color3f(r, g, b);
         // FIXME -- should we do anything with the background color?
     }
 
-    interlace_type = png_get_interlace_type (sp, ip);
+    interlace_type = png_get_interlace_type(sp, ip);
 
     // PNG files are always "unassociated alpha" but we convert to associated
     // unless requested otherwise
     if (keep_unassociated_alpha)
-        spec.attribute ("oiio:UnassociatedAlpha", (int)1);
+        spec.attribute("oiio:UnassociatedAlpha", (int)1);
 
     // FIXME -- look for an XMP packet in an iTXt chunk.
 }
@@ -265,11 +270,12 @@ read_info (png_structp& sp, png_infop& ip, int& bit_depth, int& color_type,
 /// \return empty string on success, error message on failure.
 ///
 inline const std::string
-read_into_buffer (png_structp& sp, png_infop& ip, ImageSpec& spec,
-                  int& bit_depth, int& color_type, std::vector<unsigned char>& buffer)
+read_into_buffer(png_structp& sp, png_infop& ip, ImageSpec& spec,
+                 int& bit_depth, int& color_type,
+                 std::vector<unsigned char>& buffer)
 {
     // Must call this setjmp in every function that does PNG reads
-    if (setjmp (png_jmpbuf (sp)))
+    if (setjmp(png_jmpbuf(sp)))
         return "PNG library error";
 
 #if 0
@@ -281,15 +287,15 @@ read_into_buffer (png_structp& sp, png_infop& ip, ImageSpec& spec,
     // }
 #endif
 
-    DASSERT (spec.scanline_bytes() == png_get_rowbytes(sp,ip));
-    buffer.resize (spec.image_bytes());
+    DASSERT(spec.scanline_bytes() == png_get_rowbytes(sp, ip));
+    buffer.resize(spec.image_bytes());
 
-    std::vector<unsigned char *> row_pointers (spec.height);
-    for (int i = 0;  i < spec.height;  ++i)
+    std::vector<unsigned char*> row_pointers(spec.height);
+    for (int i = 0; i < spec.height; ++i)
         row_pointers[i] = &buffer[0] + i * spec.scanline_bytes();
 
-    png_read_image (sp, &row_pointers[0]);
-    png_read_end (sp, NULL);
+    png_read_image(sp, &row_pointers[0]);
+    png_read_end(sp, NULL);
 
     // success
     return "";
@@ -301,13 +307,13 @@ read_into_buffer (png_structp& sp, png_infop& ip, ImageSpec& spec,
 /// \return empty string on success, error message on failure.
 ///
 inline const std::string
-read_next_scanline (png_structp& sp, void *buffer)
+read_next_scanline(png_structp& sp, void* buffer)
 {
     // Must call this setjmp in every function that does PNG reads
-    if (setjmp (png_jmpbuf (sp)))
+    if (setjmp(png_jmpbuf(sp)))
         return "PNG library error";
 
-    png_read_row (sp, (png_bytep)buffer, NULL);
+    png_read_row(sp, (png_bytep)buffer, NULL);
 
     // success
     return "";
@@ -318,68 +324,76 @@ read_next_scanline (png_structp& sp, void *buffer)
 /// Destroys a PNG read struct.
 ///
 inline void
-destroy_read_struct (png_structp& sp, png_infop& ip)
+destroy_read_struct(png_structp& sp, png_infop& ip)
 {
     if (sp && ip) {
-        png_destroy_read_struct (&sp, &ip, NULL);
+        png_destroy_read_struct(&sp, &ip, NULL);
         sp = NULL;
         ip = NULL;
     }
 }
 
 
+inline void
+null_png_errhandler(png_structp png, png_const_charp message)
+{
+    // ignore
+}
+
+
 
 /// Initializes a PNG write struct.
 /// \return empty string on success, C-string error message on failure.
 ///
 inline const std::string
-create_write_struct (png_structp& sp, png_infop& ip, int& color_type,
-                     ImageSpec& spec)
+create_write_struct(png_structp& sp, png_infop& ip, int& color_type,
+                    ImageSpec& spec)
 {
     // Check for things this format doesn't support
     if (spec.width < 1 || spec.height < 1)
-        return Strutil::format ("Image resolution must be at least 1x1, "
-                               "you asked for %d x %d",
-                               spec.width, spec.height);
+        return Strutil::sprintf("Image resolution must be at least 1x1, "
+                                "you asked for %d x %d",
+                                spec.width, spec.height);
     if (spec.depth < 1)
         spec.depth = 1;
     if (spec.depth > 1)
         return "PNG does not support volume images (depth > 1)";
 
     switch (spec.nchannels) {
-    case 1 :
-        color_type = PNG_COLOR_TYPE_GRAY;
+    case 1:
+        color_type         = PNG_COLOR_TYPE_GRAY;
         spec.alpha_channel = -1;
         break;
-    case 2 :
-        color_type = PNG_COLOR_TYPE_GRAY_ALPHA;
+    case 2:
+        color_type         = PNG_COLOR_TYPE_GRAY_ALPHA;
         spec.alpha_channel = 1;
         break;
-    case 3 :
-        color_type = PNG_COLOR_TYPE_RGB;
+    case 3:
+        color_type         = PNG_COLOR_TYPE_RGB;
         spec.alpha_channel = -1;
         break;
-    case 4 :
-        color_type = PNG_COLOR_TYPE_RGB_ALPHA;
+    case 4:
+        color_type         = PNG_COLOR_TYPE_RGB_ALPHA;
         spec.alpha_channel = 3;
         break;
     default:
-        return Strutil::format  ("PNG only supports 1-4 channels, not %d",
-                                 spec.nchannels);
+        return Strutil::sprintf("PNG only supports 1-4 channels, not %d",
+                                spec.nchannels);
     }
     // N.B. PNG is very rigid about the meaning of the channels, so enforce
     // which channel is alpha, that's the only way PNG can do it.
 
-    sp = png_create_write_struct (PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
-    if (! sp)
+    sp = png_create_write_struct(PNG_LIBPNG_VER_STRING, nullptr,
+                                 null_png_errhandler, nullptr);
+    if (!sp)
         return "Could not create PNG write structure";
 
-    ip = png_create_info_struct (sp);
-    if (! ip)
+    ip = png_create_info_struct(sp);
+    if (!ip)
         return "Could not create PNG info structure";
 
     // Must call this setjmp in every function that does PNG writes
-    if (setjmp (png_jmpbuf(sp)))
+    if (setjmp(png_jmpbuf(sp)))
         return "PNG library error";
 
     // success
@@ -391,8 +405,8 @@ create_write_struct (png_structp& sp, png_infop& ip, int& color_type,
 /// Helper function - writes a single parameter.
 ///
 inline bool
-put_parameter (png_structp& sp, png_infop& ip, const std::string &_name,
-               TypeDesc type, const void *data, std::vector<png_text>& text)
+put_parameter(png_structp& sp, png_infop& ip, const std::string& _name,
+              TypeDesc type, const void* data, std::vector<png_text>& text)
 {
     std::string name = _name;
 
@@ -401,32 +415,36 @@ put_parameter (png_structp& sp, png_infop& ip, const std::string &_name,
         return false;
     if (Strutil::iequals(name, "compression"))
         return false;
-    if (Strutil::iequals(name, "ResolutionUnit") ||
-          Strutil::iequals(name, "XResolution") || Strutil::iequals(name, "YResolution"))
+    if (Strutil::iequals(name, "ResolutionUnit")
+        || Strutil::iequals(name, "XResolution")
+        || Strutil::iequals(name, "YResolution"))
         return false;
 
     // Remap some names to PNG conventions
     if (Strutil::iequals(name, "Artist") && type == TypeDesc::STRING)
         name = "Author";
-    if ((Strutil::iequals(name, "name") || Strutil::iequals(name, "DocumentName")) &&
-          type == TypeDesc::STRING)
+    if ((Strutil::iequals(name, "name")
+         || Strutil::iequals(name, "DocumentName"))
+        && type == TypeDesc::STRING)
         name = "Title";
-    if ((Strutil::iequals(name, "description") || Strutil::iequals(name, "ImageDescription")) &&
-          type == TypeDesc::STRING)
+    if ((Strutil::iequals(name, "description")
+         || Strutil::iequals(name, "ImageDescription"))
+        && type == TypeDesc::STRING)
         name = "Description";
 
     if (Strutil::iequals(name, "DateTime") && type == TypeDesc::STRING) {
         png_time mod_time;
         int year, month, day, hour, minute, second;
-        if (sscanf (*(const char **)data, "%4d:%02d:%02d %02d:%02d:%02d",
-                    &year, &month, &day, &hour, &minute, &second) == 6) {
-            mod_time.year = year;
-            mod_time.month = month;
-            mod_time.day = day;
-            mod_time.hour = hour;
+        if (sscanf(*(const char**)data, "%4d:%02d:%02d %02d:%02d:%02d", &year,
+                   &month, &day, &hour, &minute, &second)
+            == 6) {
+            mod_time.year   = year;
+            mod_time.month  = month;
+            mod_time.day    = day;
+            mod_time.hour   = hour;
             mod_time.minute = minute;
             mod_time.second = second;
-            png_set_tIME (sp, ip, &mod_time);
+            png_set_tIME(sp, ip, &mod_time);
             return true;
         } else {
             return false;
@@ -462,9 +480,9 @@ put_parameter (png_structp& sp, png_infop& ip, const std::string &_name,
     if (type == TypeDesc::STRING) {
         png_text t;
         t.compression = PNG_TEXT_COMPRESSION_NONE;
-        t.key = (char *)ustring(name).c_str();
-        t.text = *(char **)data;   // Already uniquified
-        text.push_back (t);
+        t.key         = (char*)ustring(name).c_str();
+        t.text        = *(char**)data;  // Already uniquified
+        text.push_back(t);
     }
 
     return false;
@@ -475,83 +493,87 @@ put_parameter (png_structp& sp, png_infop& ip, const std::string &_name,
 /// Writes PNG header according to the ImageSpec.
 ///
 inline void
-write_info (png_structp& sp, png_infop& ip, int& color_type,
-            ImageSpec& spec, std::vector<png_text>& text,
-            bool& convert_alpha, float& gamma)
+write_info(png_structp& sp, png_infop& ip, int& color_type, ImageSpec& spec,
+           std::vector<png_text>& text, bool& convert_alpha, float& gamma)
 {
     // Force either 16 or 8 bit integers
     if (spec.format == TypeDesc::UINT8 || spec.format == TypeDesc::INT8)
-        spec.set_format (TypeDesc::UINT8);
+        spec.set_format(TypeDesc::UINT8);
     else
-        spec.set_format (TypeDesc::UINT16); // best precision available
+        spec.set_format(TypeDesc::UINT16);  // best precision available
 
-    png_set_IHDR (sp, ip, spec.width, spec.height,
-                  spec.format.size()*8, color_type, PNG_INTERLACE_NONE,
-                  PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT);
+    png_set_IHDR(sp, ip, spec.width, spec.height, spec.format.size() * 8,
+                 color_type, PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_DEFAULT,
+                 PNG_FILTER_TYPE_DEFAULT);
 
-    png_set_oFFs (sp, ip, spec.x, spec.y, PNG_OFFSET_PIXEL);
+    png_set_oFFs(sp, ip, spec.x, spec.y, PNG_OFFSET_PIXEL);
 
     // PNG specifically dictates unassociated (un-"premultiplied") alpha
-    convert_alpha = spec.alpha_channel != -1 &&
-                    !spec.get_int_attribute("oiio:UnassociatedAlpha", 0);
+    convert_alpha = spec.alpha_channel != -1
+                    && !spec.get_int_attribute("oiio:UnassociatedAlpha", 0);
 
-    gamma = spec.get_float_attribute ("oiio:Gamma", 1.0);
+    gamma = spec.get_float_attribute("oiio:Gamma", 1.0);
 
-    std::string colorspace = spec.get_string_attribute ("oiio:ColorSpace");
-    if (Strutil::iequals (colorspace, "Linear")) {
-        png_set_gAMA (sp, ip, 1.0);
-    }
-    else if (Strutil::istarts_with (colorspace, "GammaCorrected")) {
-        float g = Strutil::from_string<float>(colorspace.c_str()+14);
+    std::string colorspace = spec.get_string_attribute("oiio:ColorSpace");
+    if (Strutil::iequals(colorspace, "Linear")) {
+        png_set_gAMA(sp, ip, 1.0);
+    } else if (Strutil::istarts_with(colorspace, "GammaCorrected")) {
+        float g = Strutil::from_string<float>(colorspace.c_str() + 14);
         if (g >= 0.01f && g <= 10.0f /* sanity check */)
             gamma = g;
-        png_set_gAMA (sp, ip, 1.0f/gamma);
-    }
-    else if (Strutil::iequals (colorspace, "sRGB")) {
-        png_set_sRGB_gAMA_and_cHRM (sp, ip, PNG_sRGB_INTENT_ABSOLUTE);
+        png_set_gAMA(sp, ip, 1.0f / gamma);
+    } else if (Strutil::iequals(colorspace, "sRGB")) {
+        png_set_sRGB_gAMA_and_cHRM(sp, ip, PNG_sRGB_INTENT_ABSOLUTE);
     }
 
     // Write ICC profile, if we have anything
-    const ParamValue* icc_profile_parameter = spec.find_attribute(ICC_PROFILE_ATTR);
+    const ParamValue* icc_profile_parameter = spec.find_attribute(
+        ICC_PROFILE_ATTR);
     if (icc_profile_parameter != NULL) {
         unsigned int length = icc_profile_parameter->type().size();
 #if OIIO_LIBPNG_VERSION > 10500 /* PNG function signatures changed */
-        unsigned char *icc_profile = (unsigned char*)icc_profile_parameter->data();
+        unsigned char* icc_profile
+            = (unsigned char*)icc_profile_parameter->data();
         if (icc_profile && length)
-            png_set_iCCP (sp, ip, "Embedded Profile", 0, icc_profile, length);
+            png_set_iCCP(sp, ip, "Embedded Profile", 0, icc_profile, length);
 #else
-        char *icc_profile = (char*)icc_profile_parameter->data();
+        char* icc_profile      = (char*)icc_profile_parameter->data();
         if (icc_profile && length)
-            png_set_iCCP (sp, ip, (png_charp)"Embedded Profile", 0, icc_profile, length);
+            png_set_iCCP(sp, ip, (png_charp) "Embedded Profile", 0, icc_profile,
+                         length);
 #endif
     }
 
-    if (false && ! spec.find_attribute("DateTime")) {
+    if (false && !spec.find_attribute("DateTime")) {
         time_t now;
-        time (&now);
+        time(&now);
         struct tm mytm;
-        Sysutil::get_local_time (&now, &mytm);
-        std::string date = Strutil::format ("%4d:%02d:%02d %02d:%02d:%02d",
-                               mytm.tm_year+1900, mytm.tm_mon+1, mytm.tm_mday,
-                               mytm.tm_hour, mytm.tm_min, mytm.tm_sec);
-        spec.attribute ("DateTime", date);
+        Sysutil::get_local_time(&now, &mytm);
+        std::string date = Strutil::sprintf("%4d:%02d:%02d %02d:%02d:%02d",
+                                            mytm.tm_year + 1900,
+                                            mytm.tm_mon + 1, mytm.tm_mday,
+                                            mytm.tm_hour, mytm.tm_min,
+                                            mytm.tm_sec);
+        spec.attribute("DateTime", date);
     }
 
-    string_view unitname = spec.get_string_attribute ("ResolutionUnit");
-    float xres = spec.get_float_attribute ("XResolution");
-    float yres = spec.get_float_attribute ("YResolution");
-    float paspect = spec.get_float_attribute ("PixelAspectRatio");
+    string_view unitname = spec.get_string_attribute("ResolutionUnit");
+    float xres           = spec.get_float_attribute("XResolution");
+    float yres           = spec.get_float_attribute("YResolution");
+    float paspect        = spec.get_float_attribute("PixelAspectRatio");
     if (xres || yres || paspect || unitname.size()) {
         int unittype = PNG_RESOLUTION_UNKNOWN;
-        float scale = 1;
-        if (Strutil::iequals (unitname, "meter") || Strutil::iequals (unitname, "m"))
+        float scale  = 1;
+        if (Strutil::iequals(unitname, "meter")
+            || Strutil::iequals(unitname, "m"))
             unittype = PNG_RESOLUTION_METER;
-        else if (Strutil::iequals (unitname, "cm")) {
+        else if (Strutil::iequals(unitname, "cm")) {
             unittype = PNG_RESOLUTION_METER;
-            scale = 100;
-        } else if (Strutil::iequals (unitname, "inch") || Strutil::iequals (unitname, "in")) {
+            scale    = 100;
+        } else if (Strutil::iequals(unitname, "inch")
+                   || Strutil::iequals(unitname, "in")) {
             unittype = PNG_RESOLUTION_METER;
-            scale = 100.0/2.54;
+            scale    = 100.0 / 2.54;
         }
         if (paspect) {
             // If pixel aspect is given, allow resolution to be reset
@@ -568,23 +590,21 @@ write_info (png_structp& sp, png_infop& ip, int& color_type,
         } else if (yres == 0.0f) {
             yres = xres * (paspect ? paspect : 1.0f);
         }
-        png_set_pHYs (sp, ip, (png_uint_32)(xres*scale),
-                      (png_uint_32)(yres*scale), unittype);
+        png_set_pHYs(sp, ip, (png_uint_32)(xres * scale),
+                     (png_uint_32)(yres * scale), unittype);
     }
 
     // Deal with all other params
-    for (size_t p = 0;  p < spec.extra_attribs.size();  ++p)
-        put_parameter (sp, ip,
-                       spec.extra_attribs[p].name().string(),
-                       spec.extra_attribs[p].type(),
-                       spec.extra_attribs[p].data(),
-                       text);
+    for (size_t p = 0; p < spec.extra_attribs.size(); ++p)
+        put_parameter(sp, ip, spec.extra_attribs[p].name().string(),
+                      spec.extra_attribs[p].type(),
+                      spec.extra_attribs[p].data(), text);
 
     if (text.size())
-        png_set_text (sp, ip, &text[0], text.size());
+        png_set_text(sp, ip, &text[0], text.size());
 
-    png_write_info (sp, ip);
-    png_set_packing (sp);   // Pack 1, 2, 4 bit into bytes
+    png_write_info(sp, ip);
+    png_set_packing(sp);  // Pack 1, 2, 4 bit into bytes
 }
 
 
@@ -592,50 +612,35 @@ write_info (png_structp& sp, png_infop& ip, int& color_type,
 /// Writes a scanline.
 ///
 inline bool
-write_row (png_structp& sp, png_byte *data)
+write_row(png_structp& sp, png_byte* data)
 {
-    if (setjmp (png_jmpbuf(sp))) {
+    if (setjmp(png_jmpbuf(sp))) {
         //error ("PNG library error");
         return false;
     }
-    png_write_row (sp, data);
+    png_write_row(sp, data);
     return true;
 }
 
 
 
-/// Helper function - finalizes writing the image.
-///
+/// Helper function - finalizes writing the image and destroy the write
+/// struct.
 inline void
-finish_image (png_structp& sp)
+finish_image(png_structp& sp, png_infop& ip)
 {
     // Must call this setjmp in every function that does PNG writes
-    if (setjmp (png_jmpbuf(sp))) {
+    if (setjmp(png_jmpbuf(sp))) {
         //error ("PNG library error");
         return;
     }
-    png_write_end (sp, NULL);
+    png_write_end(sp, ip);
+    png_destroy_write_struct(&sp, &ip);
+    sp = nullptr;
+    ip = nullptr;
 }
 
 
-
-/// Destroys a PNG write struct.
-///
-inline void
-destroy_write_struct (png_structp& sp, png_infop& ip)
-{
-    if (sp && ip) {
-        finish_image (sp);
-        png_destroy_write_struct (&sp, &ip);
-        sp = NULL;
-        ip = NULL;
-    }
-}
-
-
-
 }  // namespace PNG_pvt
 
 OIIO_PLUGIN_NAMESPACE_END
-
-#endif  // OPENIMAGEIO_PNG_PVT_H
