@@ -27,91 +27,90 @@
 
   (This is the Modified BSD License)
 */
+
 #include <cstdio>
-#include <webp/decode.h>
-#include <OpenImageIO/imageio.h>
+
 #include <OpenImageIO/filesystem.h>
 #include <OpenImageIO/fmath.h>
+#include <OpenImageIO/imageio.h>
+
+#include <webp/decode.h>
 
 OIIO_PLUGIN_NAMESPACE_BEGIN
 
 namespace webp_pvt {
 
 
-class WebpInput final : public ImageInput
-{
- public:
+class WebpInput final : public ImageInput {
+public:
     WebpInput() { init(); }
     virtual ~WebpInput() { close(); }
-    virtual const char* format_name() const { return "webp"; }
-    virtual bool open (const std::string &name, ImageSpec &spec);
-    virtual bool read_native_scanline (int y, int z, void *data);
-    virtual bool close ();
+    virtual const char* format_name() const override { return "webp"; }
+    virtual bool open(const std::string& name, ImageSpec& spec) override;
+    virtual bool read_native_scanline(int subimage, int miplevel, int y, int z,
+                                      void* data) override;
+    virtual bool close() override;
 
- private:
+private:
     std::string m_filename;
-    uint8_t *m_decoded_image;
+    uint8_t* m_decoded_image;
     uint64_t m_image_size;
     long int m_scanline_size;
-    FILE *m_file;
+    FILE* m_file;
 
     void init()
     {
-        m_image_size = 0;
+        m_image_size    = 0;
         m_scanline_size = 0;
         m_decoded_image = NULL;
-        m_file = NULL;
+        m_file          = NULL;
     }
 };
 
 
 bool
-WebpInput::open (const std::string &name, ImageSpec &spec)
+WebpInput::open(const std::string& name, ImageSpec& spec)
 {
     m_filename = name;
 
     // Perform preliminary test on file type.
     if (!Filesystem::is_regular(m_filename)) {
-        error ("Not a regular file \"%s\"", m_filename.c_str());
+        error("Not a regular file \"%s\"", m_filename.c_str());
         return false;
     }
 
     // Get file size and check we've got enough data to decode WebP.
     m_image_size = Filesystem::file_size(name);
-    if (m_image_size == uint64_t(-1))
-    {
-        error ("Failed to get size for \"%s\"", m_filename);
+    if (m_image_size == uint64_t(-1)) {
+        error("Failed to get size for \"%s\"", m_filename);
         return false;
     }
-    if (m_image_size < 12)
-    {
-        error ("File size is less than WebP header for file \"%s\"", m_filename);
+    if (m_image_size < 12) {
+        error("File size is less than WebP header for file \"%s\"", m_filename);
         return false;
     }
 
     m_file = Filesystem::fopen(m_filename, "rb");
-    if (!m_file)
-    {
-        error ("Could not open file \"%s\"", m_filename.c_str());
+    if (!m_file) {
+        error("Could not open file \"%s\"", m_filename.c_str());
         return false;
     }
 
     // Read header and verify we've got WebP image.
     std::vector<uint8_t> image_header;
     image_header.resize(std::min(m_image_size, (uint64_t)64), 0);
-    size_t numRead = fread(&image_header[0], sizeof(uint8_t), image_header.size(), m_file);
-    if (numRead != image_header.size())
-    {
-        error ("Read failure for header of \"%s\" (expected %d bytes, read %d)",
-               m_filename, image_header.size(), numRead);
-        close ();
+    size_t numRead = fread(&image_header[0], sizeof(uint8_t),
+                           image_header.size(), m_file);
+    if (numRead != image_header.size()) {
+        error("Read failure for header of \"%s\" (expected %d bytes, read %d)",
+              m_filename, image_header.size(), numRead);
+        close();
         return false;
     }
 
     int width = 0, height = 0;
-    if(!WebPGetInfo(&image_header[0], image_header.size(), &width, &height))
-    {
-        error ("%s is not a WebP image file", m_filename.c_str());
+    if (!WebPGetInfo(&image_header[0], image_header.size(), &width, &height)) {
+        error("%s is not a WebP image file", m_filename.c_str());
         close();
         return false;
     }
@@ -119,24 +118,25 @@ WebpInput::open (const std::string &name, ImageSpec &spec)
     // Read actual data and decode.
     std::vector<uint8_t> encoded_image;
     encoded_image.resize(m_image_size, 0);
-    fseek (m_file, 0, SEEK_SET);
-    numRead = fread(&encoded_image[0], sizeof(uint8_t), encoded_image.size(), m_file);
-    if (numRead != encoded_image.size())
-    {
-        error ("Read failure for \"%s\" (expected %d bytes, read %d)",
-               m_filename, encoded_image.size(), numRead);
-        close ();
+    fseek(m_file, 0, SEEK_SET);
+    numRead = fread(&encoded_image[0], sizeof(uint8_t), encoded_image.size(),
+                    m_file);
+    if (numRead != encoded_image.size()) {
+        error("Read failure for \"%s\" (expected %d bytes, read %d)",
+              m_filename, encoded_image.size(), numRead);
+        close();
         return false;
     }
 
     const int CHANNEL_NUM = 4;
-    m_scanline_size = width * CHANNEL_NUM;
+    m_scanline_size       = width * CHANNEL_NUM;
     m_spec = ImageSpec(width, height, CHANNEL_NUM, TypeDesc::UINT8);
-    spec = m_spec;
+    spec   = m_spec;
 
-    if (!(m_decoded_image = WebPDecodeRGBA(&encoded_image[0], encoded_image.size(), &m_spec.width, &m_spec.height)))
-    {
-        error ("Couldn't decode %s", m_filename.c_str());
+    if (!(m_decoded_image = WebPDecodeRGBA(&encoded_image[0],
+                                           encoded_image.size(), &m_spec.width,
+                                           &m_spec.height))) {
+        error("Couldn't decode %s", m_filename.c_str());
         close();
         return false;
     }
@@ -145,47 +145,60 @@ WebpInput::open (const std::string &name, ImageSpec &spec)
 
 
 bool
-WebpInput::read_native_scanline (int y, int z, void *data)
+WebpInput::read_native_scanline(int subimage, int miplevel, int y, int z,
+                                void* data)
 {
-    if (y < 0 || y >= m_spec.width)   // out of range scanline
+    // Not necessary to lock and seek -- no subimages in Webp, and since
+    // the only thing we're doing here is a memcpy, it's already threadsafe.
+    //
+    // lock_guard lock (m_mutex);
+    // if (! seek_subimage (subimage, miplevel))
+    //     return false;
+
+    if (y < 0 || y >= m_spec.width)  // out of range scanline
         return false;
-    memcpy(data, &m_decoded_image[y*m_scanline_size], m_scanline_size);
-    return true;    
+    memcpy(data, &m_decoded_image[y * m_scanline_size], m_scanline_size);
+    return true;
 }
 
 
 bool
 WebpInput::close()
 {
-    if (m_file)
-    {
+    if (m_file) {
         fclose(m_file);
         m_file = NULL;
     }
-    if (m_decoded_image)
-    {
-        free(m_decoded_image); // this was allocated by WebPDecodeRGB and should be fread by free
+    if (m_decoded_image) {
+        free(
+            m_decoded_image);  // this was allocated by WebPDecodeRGB and should be fread by free
         m_decoded_image = NULL;
     }
     return true;
 }
 
-} // namespace webp_pvt
+}  // namespace webp_pvt
 
 // Obligatory material to make this a recognizeable imageio plugin
 OIIO_PLUGIN_EXPORTS_BEGIN
 
-    OIIO_EXPORT int webp_imageio_version = OIIO_PLUGIN_VERSION;
-    OIIO_EXPORT const char* webp_imageio_library_version () {
-        int v = WebPGetDecoderVersion();
-        return ustring::format("Webp %d.%d.%d", v>>16, (v>>8)&255, v&255).c_str();
-    }
-    OIIO_EXPORT ImageInput *webp_input_imageio_create () {
-        return new webp_pvt::WebpInput;
-    }
-    OIIO_EXPORT const char *webp_input_extensions[] = {
-        "webp", NULL
-    };
+OIIO_EXPORT int webp_imageio_version = OIIO_PLUGIN_VERSION;
+
+OIIO_EXPORT const char*
+webp_imageio_library_version()
+{
+    int v = WebPGetDecoderVersion();
+    return ustring::sprintf("Webp %d.%d.%d", v >> 16, (v >> 8) & 255, v & 255)
+        .c_str();
+}
+
+OIIO_EXPORT ImageInput*
+webp_input_imageio_create()
+{
+    return new webp_pvt::WebpInput;
+}
+
+OIIO_EXPORT const char* webp_input_extensions[] = { "webp", nullptr };
 
 OIIO_PLUGIN_EXPORTS_END
 
